package cviceni03;


//==============================================================================
/*******************************************************************************
 * Trida Sbl_Trida slouzi k prevodu celych cisel na textove retezce
 * vyjadrujici hodnotu zadaneho cisla slovy.
 *
 * @author    Rudolf Pecinovsky
 * @version   2.00.000,  11.3.2005
 */
public final class Slovy
{
//== KONSTANTNI ATRIBUTY TRIDY =================================================

    /** Nejvetsi cislo, ktere umi program prevest na text. */
    public static final long MAX = Long.MAX_VALUE;

    private static final String jednotky[] = new String[]{
        "",        "jedna",    "dva",       "tri",      "ctyri",
        "pet",     "sest",     "sedm",      "osm",      "devet",
        "deset",   "jedenact", "dvanact",   "trinact",  "ctrnact",
        "patnact", "sestnact", "sedmnact",  "osmnact",  "devatenact"
    };

    private static final String desitky[] = new String[]{
        "",        "",         "dvacet",    "tricet",   "ctyricet",
        "padesat", "sedesat",  "sedmdesat", "osmdesat", "devadesat"
    };

    private static final String stovky[] = new String[]{
        "",        "sto",      "dve ste",   "tri sta",  "ctyri sta",
        "pet set", "sest set", "sedm set",  "osm set",  "devet set"
    };

    private static final String rady[][] = {
        null,
        { "tisic",     "tisice",   "tisic"    },
        { "milion",    "miliony",  "milionu"  },
        { "miliarda",  "miliardy", "miliard"  },
        { "bilion",    "biliony",  "bilionu"  },
        { "biliarda",  "biliardy", "biliard"  },
        { "trilion",   "trliony",  "trilionu" },
    };

//== PROMENNE ATRIBUTY TRIDY ===================================================
//== KONSTANTNI ATRIBUTY INSTANCI ==============================================
//== PROMENNE ATRIBUTY INSTANCI ================================================
//== PRISTUPOVE METODY VLASTNOSTI TRIDY ========================================
//== OSTATNI METODY TRIDY ======================================================

    /***************************************************************************
     * Metoda vrati retezec predstavujici slovni vyjadreni zadaneho cisla.
     *
     * @param cislo  Cislo, ktere cheme vyjaidrit slovy.
     *
     * @return Slovni vyjadreni zadaneho cisla
     * @throws IllegalArgumentException je-li cislo prilis male nebo velike.
     */
    public static String slovy( long cislo )
    {
        if( cislo == 0 )
            return "nula";      //====================>
        if( (cislo < 0)  ||  (MAX < cislo) )
            throw new IllegalArgumentException(
                "Lze prevadet pouze nezaporna cisla do " + MAX );
        return preved( cislo );
    }

//##############################################################################
//== KONSTRUKTORY A TOVARNI METODY =============================================

    /***************************************************************************
     * Tato trida bude knihovni, bude obsahovat pouze staticke metody,
     * takze od ni zakazeme vytvaret instance.
     */
    private Slovy()    {}

//== PRISTUPOVE METODY VLASTNOSTI INSTANCI =====================================
//== PREKRYTE METODY IMPLEMENTOVANYCH ROZHRANI =================================
//== PREKRYTE ABSTRAKTNI METODY RODICOVSKE TRIDY ===============================
//== PREKRYTE KONKRETNI METODY RODICOVSKE TRIDY ================================
//== NOVE ZAVEDENE METODY INSTANCI =============================================
//== SOUKROME A POMOCNE METODY TRIDY ===========================================

    /***************************************************************************
     * Metoda vrati retezec predstavujici slovni vyjadreni zadaneho cisla.
     *
     * @param cislo  Cislo, ktere cheme vyjaidrit slovy.
     *
     * @return Slovni vyjadreni zadaneho cisla
     * @throws IllegalArgumentException je-li cislo prilis male nebo velike.
     */
    private static String stovky( int cislo )
    {
        if( cislo == 0 )
            return null;
        if( cislo < 20 )
            return jednotky[ cislo ];
        int jednotek = cislo % 10;
        int desitek  = cislo / 10;
        int stovek   = desitek / 10;
            desitek  = desitek % 10;
        return stovky[stovek] +
              (((stovek  > 0)  &&  (cislo%100 > 0))  ?  " "  :  "")  +
              ((desitek < 2)  ?  jednotky[ 10*desitek + jednotek ]
                              :  (desitky [ desitek ]) +
                                 ((jednotek > 0)  ?  " "  :  "")  +
                                 jednotky[ jednotek ]);
    }


    /***************************************************************************
     * Metoda vrati retezec predstavujici slovni vyjadreni zadaneho cisla.
     *
     * @param cislo  Cislo, ktere cheme vyjaidrit slovy.
     *
     * @return Slovni vyjadreni zadaneho cisla
     * @throws IllegalArgumentException je-li cislo prilis male nebo velike.
     */
    private static String preved( long cislo )
    {
        String[] texty = new String[ rady.length ];
        texty[0] = stovky( (int)(cislo % 1000) );
        int trojic = 1;
        while( (cislo /= 1000) > 0 )
        {
            int tricisli = (int)(cislo % 1000);
            if( tricisli == 0)
            {
                trojic++;
                continue;
            }
            String pocet = stovky( tricisli );
            String rad;
            if( tricisli == 1)
            {
                rad = rady[trojic][0];
            }
            else if( tricisli <= 3 )
            {
                rad = rady[trojic][1];
                if( (tricisli == 2)  &&
                    (trojic > 2)     &&
                    ((trojic&1) == 1) )
                //Jsou to miliardy, biliardy, ...
                {
                    pocet = "dve";
                }
            }
            else
            {
                rad = rady[trojic][2];
            }
            texty[trojic] = pocet + " " + rad;
            trojic++;
        }
        StringBuilder sb = new StringBuilder();
        for( int i = trojic-1;   i >= 0;   i-- )
        {
            if( texty[i] == null )
                continue;
            if( sb.length() > 0 )
                sb.append(" ");
            sb.append( texty[i] );
        }
        String ret = sb.toString();
        return sb.toString();
    }


//== SOUKROME A POMOCNE METODY INSTANCI ========================================
//== VNORENE A VNITRNI TRIDY ===================================================
//== TESTY A METODA MAIN =======================================================
    //+ main
    public static void main( String[] args )
    {
        long[] cislo = { 0L, 1L, 12L, 34L, 567L, 8900L, 2345L, 678000L,
            1000234L, 2000000034, 2345678901234567890L, MAX };
        for( int i = 0;   i < cislo.length;   i++ )
        {
            long c = cislo[i];
            System.out.printf( "%,d = %s\n", c, slovy( c ) );
        }
    }
    //- main
}
